/*************************

contrechoc.com 2012

example movie using GPS coordinates to plot a bicycle tour on a map of Rotterdam
the example also has functions to deal with the Dutch "Rijkscoordinaten"

one image is used for calibration, another later added to show the tour

***************************/

PImage backgroundMap, backgroundMapBIG;

float mapScreenWidth, mapScreenHeight;  // Dimension of map in pixels.

int mapShiftX = 200;//shifts the images
int mapShiftY = 300;

String[] lines;//external data file with GPS coordinates
int index = 0;

void setup() {

  size(900, 700);
  smooth();
  noLoop();

  lines = loadStrings("data1.txt");//GPS data of a bicyle tour, first from home to my studio, than along the river "Rotte"

  backgroundMap   = loadImage("rdam.png");//image for calibration
  backgroundMapBIG  = loadImage("rdamBIG.jpg");//image to get a big map

  mapScreenWidth  = width;
  mapScreenHeight = height;
  
  //we needed a bigger image of rotterdam than the calibration image, so we had to scale and shift this image to fit with the calibration image
  float scaleF1 = .77;
  image(backgroundMapBIG, mapShiftX - 200, mapShiftY - 235, scaleF1*backgroundMapBIG.width, scaleF1*backgroundMapBIG.height);//the map is shfited to the center

//this image is used for calibration...top left en bottom right points have been checked on internet
  image(backgroundMap, mapShiftX, mapShiftY);//the map is shfited to the center

  // identified two point on the map of rotterdam
  // transformed the RDM coordinates to latitude/longitude
  // drawn points on the map-image
  // given pixel width and height of the map-image the scaling can be done

  //***************************************************first calibration point Rijkscoordinaten, corner of calibration image
  float x1 = 90000;// ->>>  //51.944645 4.4418254
  float y1 = 440000; 
  //convert to latitude longitude
  float[] latLon1 = convertFunction ( x1, y1);

  float lat1 = latLon1[0];
  float lon1 = latLon1[1];

  println ( lat1 + " " + lon1 );
  //convert to image of Rotterdam
  float[] latLonScaled1 = scaleFunction2Rotterdam ( lat1, lon1);
  fill(255, 0, 0);
  ellipse( latLonScaled1[0], latLonScaled1[1], 10, 10 );

  println( latLonScaled1[0] + " " + latLonScaled1[1] );

  //******************************************************second calibration point Rijkscoordinaten, corner of calibartion image

  float x2 = 97000;
  float y2 = 433000;
  //51.882507 4.544804

  float[] latLon2 = convertFunction ( x2, y2);

  float lat2 = latLon2[0];
  float lon2 = latLon2[1];

  println ( lat2 + " " + lon2 );

  float[] latLonScaled2 = scaleFunction2Rotterdam ( lat2, lon2);
  fill(255, 255, 0);
  ellipse( latLonScaled2[0], latLonScaled2[1], 10, 10 );

  println( latLonScaled2[0] + " " + latLonScaled2[1] );

  //******************************************************end of checking point

  //now you can plot your points likewise so:
  /*
   1. get x and y of RDM
   2. conver to latitude and longitude
   3. scale the latitude and longitude
   4. plot the point on the same map
   */

  //http://itouchmap.com/latlong.html

// here we import the datafile i have recorded of a journey on bicycle along the "Rotte" river
  for (index = 0; index < lines.length; index++) {
    String[] pieces = split(lines[index], ' ');
    // println( pieces );

    float lat3 = fromDegrees2Float(float(pieces[5]));//the format is : DDMM.SSSS where SSSS are arcseconds 
    float lon3 = fromDegrees2Float(float(pieces[6]));
    float[] lonLatScaled3 = scaleFunction2Rotterdam (  lon3, lat3);
    
    fill( 255,255,0);
    if ( index%100 == 0) {
      
       fill( 255,0,0);
       text(index,   lonLatScaled3[0], lonLatScaled3[1]);
    }
    
    ellipse( lonLatScaled3[0], lonLatScaled3[1], 3, 3 );
    println ( index + " " + lon3 + " " +  lat3  + " " +  lonLatScaled3[0] + " " + lonLatScaled3[1]);
  }
  
}

float fromDegrees2Float( float myDegrees) { //to reformat the number coming from the GPS module

/**********************

 064650    5156.7751 00429.766
 time      latitude  longitude
 
**********************/

  //the value from GPS is degrees minutes seconds
  float withoutSec = 0.0 + floor ( myDegrees );//5156
  float sec = 100*(myDegrees - withoutSec);//5156.38 - 5156 = .38
  float degree = 0.0 + floor(  withoutSec/100.0 );//51
  float minutes = ( withoutSec/100.0 - degree )*100;

  return degree + minutes/60.0 + sec/6000.0;
}

void draw() {
}

float[] scaleFunction2Rotterdam (float x, float y) //lon lat
{

  //coordinates shifted with the map

  // HARDCODED: difference in latitude and longitude of the two reference points of the calibration image

  float rX = mapShiftX + ( (x  - 4.4418254)/( 4.544804 - 4.4418254 )) * backgroundMap.width; // longitude
  float rY = mapShiftY + ( 1.0-(y - 51.882507)/( 51.944645 - 51.882507 )) * backgroundMap.height; //latitude
  float[] returnArray = { 
    rX, rY
  };
  return returnArray;
}


float[] convertFunction (float x, float y) // Rijkscoordinates to longitude, latitude (same as GPS)
{
  //**********************************
  //PHP source
  // http://www.god-object.com/2009/10/23/convert-rijksdriehoekscordinaten-to-latitudelongitude/
  //**********************************

  // Calculate WGS84 coördinates
  float dX = (x - 155000) * pow(10, - 5);
  float dY = (y - 463000) * pow(10, - 5);

  float SomN = (3235.65389 * dY) + (- 32.58297 * pow(dX, 2)) + 
    (- 0.2475 * pow(dY, 2)) + (- 0.84978 * pow(dX, 2) *dY) + 
    (- 0.0655 * pow(dY, 3)) + (- 0.01709 *pow(dX, 2) * pow(dY, 2)) + 
    (- 0.00738 *dX) + (0.0053 * pow(dX, 4)) + (- 0.00039 * pow(dX, 2) * pow(dY, 3)) + 
    (0.00033 * pow(dX, 4) * dY) + (- 0.00012 *dX * dY);

  float SomE = (5260.52916 * dX) + (105.94684 * dX * dY) + (2.45656 *dX * pow(dY, 2)) + 
    (- 0.81885 * pow(dX, 3)) + (0.05594 * dX * pow(dY, 3)) + (- 0.05607 * pow(dX, 3) * dY) + (0.01199 *dY) + 
    (- 0.00256 * pow(dX, 3) * pow( dY, 2)) + (0.00128 * dX * pow(dY, 4)) + 
    (0.00022 * pow(dY, 2)) + (- 0.00022 * pow( dX, 2)) + (0.00026 * pow(dX, 5));

  float Latitude = 52.15517 + (SomN / 3600);
  float Longitude = 5.387206 + (SomE / 3600);

  float[] latLong = {  
    Latitude, Longitude
  };
  return latLong;
}

